// CGMScriptView.h : interface of the CCGMScriptView class
//
/////////////////////////////////////////////////////////////////////////////

#include "ElemTree.h"	// Tree Control for Programming Elements

class CCGMScriptView : public CFormView
{
protected: // create from serialization only
	CCGMScriptView();
	DECLARE_DYNCREATE(CCGMScriptView)

public:
	//{{AFX_DATA(CCGMScriptView)
	enum { IDD = IDD_CGMSCRIPT_FORM };
	//}}AFX_DATA
	
	// Members for controls added dynamically
	CElemTree	m_ElemTree;			// Element Tree
	CListBox	m_ItemList;			// Script Item List
	CImageList	m_ImageList;		// Icon list for Tree Ctrl
	COleDropTarget	m_OleTarget;	// For OLE Drag & Drop
	
// Attributes
public:
	CCGMScriptDoc* GetDocument();
	int		m_nPage;	// Page currently printing
	
// Operations
	inline void SubmitTask(CTask* pTask)
		{ GetDocument()->m_Foreman.SubmitTask(pTask); }

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCGMScriptView)
	public:
	virtual void OnInitialUpdate();
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual BOOL OnPreparePrinting(CPrintInfo* pInfo);
	virtual void OnBeginPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnEndPrinting(CDC* pDC, CPrintInfo* pInfo);
	virtual void OnPrint(CDC* pDC, CPrintInfo*);
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual ~CCGMScriptView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

private:
	// GUI Services
	void UpdateList(CCGMScriptDoc*	pDoc,BOOL bDoRedo, CTask* pTask);		// Draws List Box from scratch

	// Editing primitives
	void AddElem(int i, BOOL where);	// Add an Element at Selection
	void DelElem(int i);				// Delete Selected Element(s);
	void EditElem(int i);				// Edit Selected Element(s);

	// Commands
	void Comment();						// Comment Selection
	void Indent();						// Indent Selection
	void Undent();						// Outdent Selection
	void Delete();						// Delete Selection
	void Generate();					// Generate File
	void EditSel();						// Edit Selection


// Generated message map functions
protected:
	//{{AFX_MSG(CCGMScriptView)
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg void OnItemexpandingElemtree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDblclkElemtree(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnDblclkScript();
	afx_msg void OnCmtBtn();
	afx_msg void OnUpdateCmtBtn(CCmdUI* pCmdUI);
	afx_msg void OnDelBtn();
	afx_msg void OnUpdateDelBtn(CCmdUI* pCmdUI);
	afx_msg void OnEdtBtn();
	afx_msg void OnUpdateEdtBtn(CCmdUI* pCmdUI);
	afx_msg void OnIndBtn();
	afx_msg void OnUpdateIndBtn(CCmdUI* pCmdUI);
	afx_msg void OnUndBtn();
	afx_msg void OnUpdateUndBtn(CCmdUI* pCmdUI);
	afx_msg void OnGenBtn();
	afx_msg void OnUpdateGenBtn(CCmdUI* pCmdUI);
	afx_msg void OnEditCom();
	afx_msg void OnUpdateEditCom(CCmdUI* pCmdUI);
	afx_msg void OnEditDel();
	afx_msg void OnUpdateEditDel(CCmdUI* pCmdUI);
	afx_msg void OnEditGen();
	afx_msg void OnUpdateEditGen(CCmdUI* pCmdUI);
	afx_msg void OnEditInd();
	afx_msg void OnUpdateEditInd(CCmdUI* pCmdUI);
	afx_msg void OnEditUnd();
	afx_msg void OnUpdateEditUnd(CCmdUI* pCmdUI);
	afx_msg void OnAddBtn();
	afx_msg void OnUpdateAddBtn(CCmdUI* pCmdUI);
	afx_msg LRESULT OnDragMessage(WPARAM wParam, LPARAM lParam);
	afx_msg void OnEditUndo();
	afx_msg void OnUpdateEditUndo(CCmdUI* pCmdUI);
	afx_msg void OnEditRedo();
	afx_msg void OnUpdateEditRedo(CCmdUI* pCmdUI);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
	
};

/////////////////////////////////////////////////////////////////////
/// Inline declarations

#ifndef _DEBUG  // debug version in CGMScriptView.cpp
inline CCGMScriptDoc* CCGMScriptView::GetDocument()
   { return (CCGMScriptDoc*)m_pDocument; }
#endif

inline CCGMScriptView::~CCGMScriptView() { /* empty */ }
inline void CCGMScriptView::OnBeginPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/) { /* empty */  }
inline void CCGMScriptView::OnEndPrinting(CDC* /*pDC*/, CPrintInfo* /*pInfo*/) { /* empty */ }
inline void CCGMScriptView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
	{ UpdateList(GetDocument(),(BOOL) lHint, (CTask*) pHint); }

// Comment Selection
inline void CCGMScriptView::OnCmtBtn() 
	{ Comment(); }
// Delete Selection
inline void CCGMScriptView::OnDelBtn() 
	{ Delete(); }
// Edit Selection
inline void CCGMScriptView::OnEdtBtn() 
	{ EditSel(); }
// Indent Selection
inline void CCGMScriptView::OnIndBtn() 
	{ Indent(); }
// Outdent Selection
inline void CCGMScriptView::OnUndBtn() 
	{ Undent(); }
// Generate File 
inline void CCGMScriptView::OnGenBtn()	
	{ Generate(); }
// Comment Menu action
inline void CCGMScriptView::OnEditCom()	
	{ Comment(); }
// Delete Menu action
inline void CCGMScriptView::OnEditDel()	
	{ Delete(); }
// Run Menu action
inline void CCGMScriptView::OnEditGen() 
	{ Generate(); }
// Indent Menu action
inline void CCGMScriptView::OnEditInd() 
	{ Indent(); }
// Outdent Menu action
inline void CCGMScriptView::OnEditUnd() 
	{ Undent(); }

// Undo action
inline void CCGMScriptView::OnEditUndo() 
	{ GetDocument()->m_Foreman.Undo(); }
// Redo action
inline void CCGMScriptView::OnEditRedo() 
	{ GetDocument()->m_Foreman.Redo(); }

/////////////////////////////////////////////////////////////////////////////
