/*
 * foreman.cpp - Implementation of undo history manager
 */

#include "stdafx.h"
#include "CGMScript.h"

#include "Foreman.h"
#include "CGMScriptDoc.h"
#include "Task.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/*
 * Foreman
 */

CForeman::CForeman()
{
	m_iDoUndoIndex = 0;
}

CForeman::~CForeman()
{
	// Destroy all tasks before destryoing the foreman
	while (m_taskList.GetCount() > 0)
		delete m_taskList.RemoveTail();
}

void CForeman::ClearDocument()
{
	// Destroy all tasks
	while (m_taskList.GetCount() > 0)
		delete m_taskList.RemoveTail();
	m_iDoUndoIndex = 0;
}

void CForeman::SubmitTask(CTask* task)
{
	// Upon submitting a *new* task, all 'redo-able' actions must be deleted
	while (m_iDoUndoIndex < m_taskList.GetCount())
		delete m_taskList.RemoveTail();
	m_DoUndoPosition = m_taskList.AddTail(task);
	m_iDoUndoIndex++;
	task->Do(m_pDoc);
	m_pDoc->SetModifiedFlag();
	m_pDoc->UpdateAllViews(NULL,TRUE,task);
}

void CForeman::Undo()
{
	if (CanUndo()) {
		// Undo the current entry, then backup in the list.

		// GetPrev returns the object at the *current* position
		// indicated by m_DoUndoPosition, then sets m_DoUndoPosition
		// back one link.  A better name is "GetCurrentAndMoveBackPosition"
		CTask* pTask = (CTask *) m_taskList.GetPrev(m_DoUndoPosition);
		--m_iDoUndoIndex;
		pTask->Undo(m_pDoc);
		m_pDoc->UpdateAllViews(NULL,FALSE,pTask);
	}

	// If the user undid everything, then the document is unchanged
	// relative to how it started.  Clear the "modified" flag to show this.
	if (m_iDoUndoIndex == 0) m_pDoc->SetModifiedFlag(FALSE);
}

void CForeman::Redo()
{
	if (CanRedo()) {
		// Advance to the next entry and execute it.
		m_iDoUndoIndex++;
		if (!m_DoUndoPosition)
			m_DoUndoPosition = m_taskList.GetHeadPosition();
		else
			m_taskList.GetNext(m_DoUndoPosition);
		CTask* pTask = (CTask *) m_taskList.GetAt(m_DoUndoPosition);
		pTask->Do(m_pDoc);
		m_pDoc->SetModifiedFlag();
		m_pDoc->UpdateAllViews(NULL,TRUE,pTask);
	}
}

LPCSTR CForeman::GetRedoDescription()
{
	if (CanRedo()) {
		POSITION nextPos = m_DoUndoPosition;
		if (!nextPos)
			nextPos = m_taskList.GetHeadPosition();
		else
			m_taskList.GetNext(nextPos);
		return ((CTask *) m_taskList.GetAt(nextPos))->GetDescription();
	}
	return "";
}

LPCSTR CForeman::GetUndoDescription()
{
	if (CanUndo()) {
		return ((CTask *) m_taskList.GetAt(m_DoUndoPosition))->GetDescription();
	}
	return "";
}

