/*
 * foreman.h - Class definition of undo history manager
 */

// Forward definitions.
// By having these we avoid having to load include files.
class CTask;
class CCGMScriptDoc;

////////////////////////
// Type definitions for Task and Foreman classes
//typedef CTypedPtrList<CObList,CTask *> CTaskList;
typedef CObList CTaskList;
////////////////////////////
// CForeman Class Definition

class CForeman
{
public:
	CForeman();
	~CForeman();
	void SubmitTask(CTask*);
	void ClearDocument();
	void Undo();
	void Redo();

	// The "Can" functions are used by OnCommandUpdate
	// to figure out when to grey out the menu option.
	BOOL CanRedo();
	BOOL CanUndo();

	// The "GetDescription" functions are used by OnCommand
	// to describe the actions waiting to be redone and undone.
	LPCSTR GetRedoDescription();
	LPCSTR GetUndoDescription();

	// The foreman is an aggregate member of the document.
	// SetDocument is called by the document's ctor to
	// provide a pointer back to the doc.
	void SetDocument(CCGMScriptDoc* pDoc);

private:
	//
	// Linked list of tasks.
	CTaskList	m_taskList;

	// The position and the index both indicate the next
	// task to undo, but in slightly different forms.
	POSITION	m_DoUndoPosition;
	int			m_iDoUndoIndex;

	// Remember which document this foreman is related to.
	CCGMScriptDoc*	m_pDoc;
};

inline BOOL CForeman::CanRedo() { return m_iDoUndoIndex < m_taskList.GetCount(); }
inline BOOL CForeman::CanUndo() { return m_iDoUndoIndex > 0; }
inline void CForeman::SetDocument(CCGMScriptDoc* pDoc) { m_pDoc = pDoc; }

