// MFElem.h : header file
//
// Class CMFElem implements a generic Scripting Element Object from which 
// all other objects can be derived. The parent class is CDialog 
// because it eliminates the need of a storage class and a dialog 
// class for each Element. Element Data is stored as Dialog data.
// Each Derived class should implement the following:
//	- Serialize (and use the DECLARE_SERIAL/IMPLEMENT_SERIAL macros).
//	- The Assignment operator. (for Undo/Redo).
//	- GetString (for display in list).
//	- Generate (for metafile generation).
//
// Elements that have no data can rely on the base class for Serialize.
// The assignment operator is needed for the Undo/Redo feature in
// the view class. When undoing or redoing modifications the element
// is copied in the undo list. 
/////////////////////////////////////////////////////////////////////////////
// CMFElem dialog

class CCGMScriptDoc;

class CMFElem : public CDialog
{
// Construction
public:
	CMFElem(CWnd* pParent = NULL);   // standard constructor
	CMFElem(UINT dId, CWnd* pParent = NULL);   // overloaded for dialog ID
	DECLARE_SERIAL(CMFElem)

// Dialog Data
	//{{AFX_DATA(CMFElem)
	enum { IDD = IDD_DLG };
		// NOTE: the ClassWizard will add data members here
	//}}AFX_DATA
	int n_Indent;		// Keeps track of the indentation level for the string.
	BOOL b_Comment;		// If element is commented out

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMFElem)
	public:
	virtual void Serialize(CArchive& ar);
	virtual int DoModal();
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
public:
	virtual CString GetString();	// For display
	virtual long Generate(CCGMScriptDoc *);			// For generation
	virtual CMFElem& operator=(const CMFElem&);	// Assignment operator for Undo/Redo

protected:

	// Generated message map functions
	//{{AFX_MSG(CMFElem)
		// NOTE: the ClassWizard will add member functions here
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};


