/*
 * task.cpp - Implementation of undo/redo tasks
 */
#include "stdafx.h"
#include "CGMScript.h"

#include "Foreman.h"
#include "CGMScriptDoc.h"
#include "Task.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNAMIC(CTask, CObject)
/*
 * Undo/Redo Actions
 */

/////////////////////////////////////

IMPLEMENT_DYNAMIC(CClearAllTask, CTask)

CClearAllTask::~CClearAllTask()
{
	int i=0;
	for(i=0;i<m_ElemArray.GetSize();i++)
		delete m_ElemArray[i];				// Empty CMFElem Array
	m_ElemArray.RemoveAll();				// Reset Array Size.
}

void CClearAllTask::Do(CCGMScriptDoc* pDoc)
{
	CObArray& documentList = pDoc->m_ElemArray;
	int max= pDoc->m_ElemArray.GetSize();
	int i=0;
	// Copy all objects to local array
	for(i=0;i<max;i++)	m_ElemArray.SetAtGrow(i, documentList[i]);
	// Empty document array
	documentList.RemoveAll();
}

void CClearAllTask::Undo(CCGMScriptDoc* pDoc)
{
	CObArray& documentList = pDoc->m_ElemArray;
	int max= pDoc->m_ElemArray.GetSize();
	int i=0;
	// Copy from local array to document
	for(i=0;i<max;i++) documentList.SetAtGrow(i, m_ElemArray[i]);
	// Empty local array
	m_ElemArray.RemoveAll();
}

/////////////////////////////////////

IMPLEMENT_DYNAMIC(CDelElemTask, CTask)

void CDelElemTask::Do(CCGMScriptDoc* pDoc)
{
	CObArray& m_ElemArray = pDoc->m_ElemArray;
	m_pElem = (CMFElem *) m_ElemArray[m_Pos];		// Make a local copy
	m_ElemArray.RemoveAt(m_Pos,1);					// Remove element from array
}

void CDelElemTask::Undo(CCGMScriptDoc* pDoc)
{
	pDoc->m_ElemArray.InsertAt(m_Pos, m_pElem);		// Insert local copy in array
	m_pElem = NULL;									// Void the local copy
}

/////////////////////////////////////

IMPLEMENT_DYNAMIC(CAddElemTask, CTask)

void CAddElemTask::Do(CCGMScriptDoc* pDoc)
{
	CObArray& m_ElemArray = pDoc->m_ElemArray;
	if (m_Pos == m_ElemArray.GetSize())
		m_ElemArray.Add(m_pElem);				// Append local copy to array
	else
		m_ElemArray.InsertAt(m_Pos,m_pElem);	// Insert local copy in array
	m_pElem = NULL;									// Void the local copy
}

void CAddElemTask::Undo(CCGMScriptDoc* pDoc)
{
	CObArray& m_ElemArray = pDoc->m_ElemArray;
	m_pElem = (CMFElem *) m_ElemArray[m_Pos];		// Make a local copy
	m_ElemArray.RemoveAt(m_Pos,1);					// Remove element from array
}

/////////////////////////////////////

IMPLEMENT_DYNAMIC(CModElemTask, CTask)

void CModElemTask::Do(CCGMScriptDoc* pDoc)
{
	CObArray& m_ElemArray = pDoc->m_ElemArray;
	// Swap the element in Array with element in Task
	CMFElem* pMFElem = (CMFElem *) m_ElemArray[m_Pos];
	m_ElemArray[m_Pos] = m_pElem;
	m_pElem = pMFElem;
}

void CModElemTask::Undo(CCGMScriptDoc* pDoc)
{
	Do(pDoc);									// Undo is just like Do !
}

