//
// task.h - Definitions of Undo/Redo Tasks
//
// NOTES
//
// 1. Strings should be in resources, but leaving them
//    here makes the examples clearer.
// 2. The virtual functions are not really inline,
//    but they are so short that it makes it easier to read.
//
/////////////////////////////////////////////////////////////////////////////

class CTask : public CObject {
public:
	CTask()							{ /* empty */ }
	DECLARE_DYNAMIC(CTask)
	virtual ~CTask()				{ /* empty */ }
	virtual void Do(CCGMScriptDoc*) = 0;
	virtual void Undo(CCGMScriptDoc*) = 0;
	virtual LPCSTR GetDescription() = 0;
	int		m_Pos;		// Needed for Do()

private:
	CTask(const CTask&);			// Disable copy constructor
	CTask& operator=(const CTask&);	// Disable assignment operator

protected:
	CString		m_Desc;				// Needed for GetDescription()
};

/////////////////////////////
////  Derived classes 

class CDelElemTask : public CTask {
public:
	CDelElemTask(int pos)		{ m_Pos = pos; m_pElem=NULL;}
	DECLARE_DYNAMIC(CDelElemTask)
	virtual ~CDelElemTask()			{ delete m_pElem; }
	virtual void Do(CCGMScriptDoc*);
	virtual void Undo(CCGMScriptDoc*);
	virtual LPCSTR GetDescription()		{ return "Delete"; }
private:
	CMFElem*	m_pElem;	// Needed for Undo()
};

class CClearAllTask : public CTask {
public:
	CClearAllTask()						{ /* empty */ }
	DECLARE_DYNAMIC(CClearAllTask)
	virtual ~CClearAllTask();
	virtual void Do(CCGMScriptDoc*);
	virtual void Undo(CCGMScriptDoc*);
	virtual LPCSTR GetDescription() { return m_Desc; }
private:
	CObArray	m_ElemArray;   // Needed for Undo()
};

class CAddElemTask : public CTask {
public:
	CAddElemTask(int pos, CMFElem* elem, char* desc)	{ m_Pos = pos; m_pElem=elem; m_Desc=desc;}
	DECLARE_DYNAMIC(CAddElemTask)
	virtual ~CAddElemTask()			{ delete m_pElem; }
	virtual void Do(CCGMScriptDoc*);
	virtual void Undo(CCGMScriptDoc*);
	virtual LPCSTR GetDescription()		{ return m_Desc; }
private:
	CMFElem*	m_pElem;	// Needed for Undo()
};

class CModElemTask : public CTask {
public:
	CModElemTask(int pos, CMFElem* elem, char* desc)	{ m_Pos = pos; m_pElem=elem; m_Desc=desc;}
	DECLARE_DYNAMIC(CModElemTask)
	virtual ~CModElemTask()			{ delete m_pElem; }
	virtual void Do(CCGMScriptDoc*);
	virtual void Undo(CCGMScriptDoc*);
	virtual LPCSTR GetDescription()		{ return m_Desc; }
private:
	CMFElem*	m_pElem;	// Needed for Undo()
};

