#include "stdafx.h"
#include "CGMScript.h"

#include "Foreman.h"
#include "CGMScriptDoc.h"
#include "CGMScriptView.h"
#include "Task.h"

#include <math.h>
#include <ctype.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

///////////////////////////////////////////////////////////////////////////
/// CParser implementation

// Constructor
CParser::CParser(CCGMScriptDoc *doc,const char *s_macro)
{
	yytext=NULL;
	yylast=NULL;
	yyleng=0;
	lookahead=NO_TOKEN;
	inbuf = new char[LINE_LEN];
	strncpy(inbuf,s_macro,max(LINE_LEN,strlen(s_macro)));
	p = inbuf;
	sp=0;
	result[0]=INVALID_MACRO;
	pDoc= doc;
}

int CParser::yylex(void){
	int ret;

	if (!*p) return EOI;
	while(isspace(*p)) p++; /* skip spaces */

	yytext=p;yyleng=1;
	/* Always point to next symbol before returning ! */
	/* Front line tests do not increment p */
	/* Terminal Symbols */
	if ((*p == '+') || (*p == '-') || (*p == '(') || (*p == ')') || 
		(*p == ',') || (*p == '.') || (*p == '*') || (*p == '/') || 
		(*p == '%') ) 
		ret= *p++;			/* return symbol and point to next char */
	/* Integers & Hex values */
	else if (isdigit(*p)) {
			p++;
			/* Hex value must start with 0x */
			if (*(p-1) == '0' && *p == 'x') {
				p++;yyleng=1;
				if (isxdigit(*p++)) {
					while(isxdigit(*p)) {yyleng++;p++;}
					ret = HEXVAL;
				}
				else ret = NO_TOKEN;
			}
			else {
				while(isdigit(*p)) {yyleng++;p++;}
				ret = INTEGER;
			}
		}
	/* macros start with & and a letter, then a-z 0-9 */
	else if (*p == '&') {
		p++;
		if (isalpha(*p++)) {
			while (isalnum(*p)) {yyleng++;p++;}
			yytext++;	/* Skip the ampersand */
			ret = NAME;
		}
		else ret = NO_TOKEN;
	}
	/* all other terminals */
	else if (isalpha(*p)) {
		p++;
		while (isalpha(*p)) {yyleng++;p++;}
		/* Check that it matches any of the known functions */
		switch (yyleng) {
			case 1:
				if ((*yytext == 'E') ||
					(*yytext == 'e')) ret = 'E';
				else ret = NO_TOKEN;
				break;
			case 2:
				if (!strncmp(yytext,"ln",2)) ret = LN;
				else if (!strncmp(yytext,"lg",2)) ret = LG;
				else if (!strncmp(yytext,"or",2)) ret = OR;
				else ret = NO_TOKEN;
				break;
			case 3:
				if (!strncmp(yytext,"Anp",3)) ret = ANP;
				else if (!strncmp(yytext,"pow",3)) ret = POW;
				else if (!strncmp(yytext,"abs",3)) ret = ABS;
				else if (!strncmp(yytext,"and",3)) ret = AND;
				else if (!strncmp(yytext,"Cnp",3)) ret = CNP;
				else if (!strncmp(yytext,"cos",3)) ret = COS;
				else if (!strncmp(yytext,"div",3)) ret = DIV;
				else if (!strncmp(yytext,"exp",3)) ret = EXP;
				else if (!strncmp(yytext,"log",3)) ret = LOG;
				else if (!strncmp(yytext,"not",3)) ret = NOT;
				else if (!strncmp(yytext,"sin",3)) ret = SIN;
				else if (!strncmp(yytext,"tan",3)) ret = TAN;
				else if (!strncmp(yytext,"xor",3)) ret = XOR;
				else ret = NO_TOKEN;
				break;
			case 4:
				if (!strncmp(yytext,"asin",4)) ret = ASIN;
				else if (!strncmp(yytext,"acos",4)) ret = ACOS;
				else if (!strncmp(yytext,"atan",4)) ret = ATAN;
			default:
				ret = NO_TOKEN;
				break;
		}
	}
	else ret= NO_TOKEN;
	return ret;
}
/* match a token */
int CParser::match(int t){
	if(NO_TOKEN==lookahead)
		lookahead=yylex();
	return lookahead==t;
}
/* advance the lookahead to the next token */
void CParser::next(void){
	yylast = yytext;
	lookahead=yylex();
}

int CParser::Eexp_parse(void){
	if((!match('+'))&&(!match('-'))&&(!match(INTEGER))){
		yyerror("Eexp");
		return 1;
	}
	else if(match('+')){
		Eexp_action('+');
		next();
		if(match(INTEGER)) {
		Eexp_action(INTEGER);
			next();
		}
		else{
			yyerror("INTEGER");
			return 1;
		}
	}
	else if(match('-')){
		Eexp_action('-');
		next();
		if(match(INTEGER)) {
		Eexp_action(INTEGER);
			next();
		}
		else{
			yyerror("INTEGER");
			return 1;
		}
	}
	else if(match(INTEGER)){
		Eexp_action(INTEGER);
		next();
	}
	else return 1;
	Eexp_action(0);
	return 0;
}

int CParser::Eexp_action(int token){
	static double sign = 1.0;
	char temp[MAX_LEN];
	switch (token) {
		case '-':
			sign = -1.0;
			break;
		case '+':
			sign = +1.0;
			break;
		case INTEGER:
			strncpy(temp,yytext,max(yyleng,MAX_LEN));
			if (++sp < MAX_DEPTH) result[sp] = atol(temp);
				else throw IDS_PARSER_STACK_OVF;
			break;
		case 0:
			result[sp] *= sign;
			sign = +1.0;
			break;
	}
	return 0;
}

int CParser::exponent_parse(void){
	if((!match('E'))&&(!match(AND))&&(!match('%'))&&(!match('/'))&&(!match('*'))&&(!match(XOR))&&(!match(OR))&&(!match('-'))&&(!match('+'))&&(!match(','))&&(!match(')'))&&(!match(EOI))){
		yyerror("exponent");
	}
	else if(match('E')){
		exponent_action('E');
		next();
		if(Eexp_parse()){
			yyerror("Eexp");
			return 1;
		}
	}
	else if(match(AND)||match('%')||match('/')||match('*')||match(XOR)||match(OR)||match('-')||match('+')||match(',')||match(')')||match(EOI)){
		return 0;
	}
	else return 1;
	exponent_action(0);
	return 0;
}

int CParser::exponent_action(int token){
	switch (token) {
		case 'E':
			break;
		case 0:
			/* Multiply the current float by this power of 10 */
			sp--;
			result[sp] = result[sp] * pow(10.0,result[sp+1]);
			break;
	}
	return 0;
}

int CParser::decimal_parse(void){
	if((!match('.'))&&(!match('E'))&&(!match(AND))&&(!match('%'))&&(!match('/'))&&(!match('*'))&&(!match(XOR))&&(!match(OR))&&(!match('-'))&&(!match('+'))&&(!match(','))&&(!match(')'))&&(!match(EOI))){
		yyerror("decimal");
	}
	else if(match('.')){
		decimal_action('.');
		next();
		if(match(INTEGER)) {
		decimal_action(INTEGER);
			next();
		}
		else{
			yyerror("INTEGER");
			return 1;
		}
	}
	else if(match('E')||match(AND)||match('%')||match('/')||match('*')||match(XOR)||match(OR)||match('-')||match('+')||match(',')||match(')')||match(EOI)){
		return 0;
	}
	else return 1;
	decimal_action(0);
	return 0;
}

int CParser::decimal_action(int token){
	char temp[MAX_LEN];
	static int power;
	switch (token) {
		case '.':
			break;
		case INTEGER:
			strncpy(temp,yytext,max(yyleng,MAX_LEN));
			power = yyleng;
			if (++sp < MAX_DEPTH) result[sp] = atol(temp);
				else throw IDS_PARSER_STACK_OVF;
			break;
		case 0:
			/* Add the decimal part to current integer */
			sp--;
			for(;power;power--)
				result[sp+1] = result[sp+1]/10.0;
			result[sp] = result[sp] + result[sp+1];
			break;
	}
	return 0;
}

int CParser::afloat_parse(void){
	if((!match(INTEGER))){
		yyerror("afloat");
	}
	else if(match(INTEGER)){
		afloat_action(INTEGER);
		next();
		if(decimal_parse()){
			yyerror("decimal");
			return 1;
		}
		if(exponent_parse()){
			yyerror("exponent");
			return 1;
		}
	}
	else return 1;
	afloat_action(0);
	return 0;
}

int CParser::afloat_action(int token){
	char temp[MAX_LEN];
	if (token == INTEGER) {
		strncpy(temp,yytext,max(yyleng,MAX_LEN));
		if (++sp < MAX_DEPTH) result[sp] = atol(temp);
			else throw IDS_PARSER_STACK_OVF;
	}
	return 0;
}

int CParser::number_parse(void){
	if((!match(NAME))&&(!match('-'))&&(!match('+'))&&(!match(INTEGER))&&(!match(HEXVAL))){
		yyerror("number");
	}
	else if(match(NAME)){
		number_action(NAME);
		next();
	}
	else if(match('-')){
		number_action('-');
		next();
		if(afloat_parse()){
			yyerror("afloat");
			return 1;
		}
	}
	else if(match('+')){
		number_action('+');
		next();
		if(afloat_parse()){
			yyerror("afloat");
			return 1;
		}
	}
	else if(match(INTEGER)){
		if(afloat_parse()){
			yyerror("afloat");
			return 1;
		}
	}
	else if(match(HEXVAL)){
		number_action(HEXVAL);
		next();
	}
	else return 1;
	number_action(0);
	return 0;
}

int CParser::number_action(int token){
	static double sign=1.0;
	char *temp;
	switch (token) {
		case '-':
			sign = -1.0;
		case '+':
			sign = +1.0;
			break;
		case NAME:
			// This case actually resolves the value of a global
			// macro (at the document level) if the cuurent macro
			// is being evaluated. Otherwise, just OK the syntax.
			if (++sp < MAX_DEPTH) {
				if (resolve && pDoc != NULL) {
					// We are actually evaluating the value of a macro
					CMacro *pMacro = pDoc->FindByName(yytext,yyleng);
					if (pMacro !=NULL)
						result[sp] = pMacro->resolve(pDoc);
					else 
						throw IDS_UNDEFINED_VAR;
				}
				else result[sp] = 0.0;
			}
			else throw IDS_PARSER_STACK_OVF;
			break;
		case HEXVAL:
			/* Hex to Decimal conversion */
			if (++sp < MAX_DEPTH) result[sp] = 0.0;
				else throw IDS_PARSER_STACK_OVF;
			temp = yytext+2;
			{
				long hexnum=0;
				for(;yyleng;yyleng--,temp++) 
					hexnum = (hexnum << 0x04) | (*temp - 0x30);
				result[sp] = (double) hexnum;
			}
			break;
		case 0:
			result[sp] *= sign;
			sign = +1.0;
			break;
	}
	return 0;
}

int CParser::Zstmt_parse(void){
	if((!match(AND))&&(!match('%'))&&(!match('/'))&&(!match('*'))&&(!match(XOR))&&(!match(OR))&&(!match('-'))&&(!match('+'))&&(!match(','))&&(!match(')'))&&(!match(EOI))){
		yyerror("Zstmt");
	}
	else if(match(AND)){
		Zstmt_action(AND);
		next();
		if(term_parse()){
			yyerror("term");
			return 1;
		}
	}
	else if(match('%')){
		Zstmt_action('%');
		next();
		if(term_parse()){
			yyerror("term");
			return 1;
		}
	}
	else if(match('/')){
		Zstmt_action('/');
		next();
		if(term_parse()){
			yyerror("term");
			return 1;
		}
	}
	else if(match('*')){
		Zstmt_action('*');
		next();
		if(term_parse()){
			yyerror("term");
			return 1;
		}
	}
	else if(match(XOR)||match(OR)||match('-')||match('+')||match(',')||match(')')||match(EOI)){
		return 0;
	}
	else return 1;
	Zstmt_action(0);
	return 0;
}

int CParser::Zstmt_action(int token){
	if (token)
		result[++sp] = token;
	else {
		/* combine the result of term and Zstmt */
		sp -= 2;
		switch ((int) result[sp+1]) {
			case '*':
				result[sp] = result[sp] * result[sp+2];
				break;
			case '/':
				result[sp] = result[sp] / result[sp+2];
				break;
			case '%':
				result[sp] = (int) result[sp] % (int) result[sp+2];
				break;
			case AND:
				result[sp] = (int) result[sp] & (int) result[sp+2];
				break;
		}
	}
	return 0;
}

int CParser::term_parse(void){
	if((!match(ANP))&&(!match(POW))&&(!match(CNP))&&(!match(DIV))&&(!match(NOT))&&(!match(TAN))&&(!match(COS))&&(!match(ASIN))&&(!match(ACOS))&&(!match(ATAN))&&(!match(ABS))&&(!match(SIN))&&(!match(EXP))&&(!match(LG))&&(!match(LN))&&(!match(LOG))&&(!match('('))&&(!match(NAME))&&(!match('-'))&&(!match('+'))&&(!match(INTEGER))&&(!match(HEXVAL))){
		yyerror("term");
	}
	else if(match(ANP)){
		term_action(ANP);
		goto L_next_bin;
	}
	else if(match(POW)){
		term_action(POW);
		goto L_next_bin;
	}
	else if(match(CNP)){
		term_action(CNP);
		goto L_next_bin;
	}
	else if(match(DIV)){
		term_action(DIV);
		goto L_next_bin;
	}
	else if(match(NOT)){
		term_action(NOT);
		goto L_next_un;
	}
	else if(match(TAN)){
		term_action(TAN);
		goto L_next_un;
	}
	else if(match(COS)){
		term_action(COS);
		goto L_next_un;
	}
	else if(match(SIN)){
		term_action(SIN);
		goto L_next_un;
	}
	else if(match(ABS)){
		term_action(ABS);
		goto L_next_un;
	}
	else if(match(ASIN)){
		term_action(ASIN);
		goto L_next_un;
	}
	else if(match(ACOS)){
		term_action(ACOS);
		goto L_next_un;
	}
	else if(match(ATAN)){
		term_action(ATAN);
		goto L_next_un;
	}
	else if(match(EXP)){
		term_action(EXP);
		goto L_next_un;
	}
	else if(match(LG)){
		term_action(LG);
		goto L_next_un;
	}
	else if(match(LN)){
		term_action(LN);
		goto L_next_un;
	}
	else if(match(LOG)){
		term_action(LOG);
		goto L_next_un;
	}
	else if(match('(')){
		term_action('(');
		next();
		if(expr_parse()){
			yyerror("expr");
			return 1;
		}
		if(match(')')) {
		term_action(')');
			next();
		}
		else{
			yyerror("')'");
			return 1;
		}
	}
	else if(match(NAME)||match('-')||match('+')||match(INTEGER)||match(HEXVAL)){
		if(number_parse()){
			yyerror("number");
			return 1;
		}
	}
	else return 1;
	term_action(0);
	return 0;

L_next_un:
	next();
	if(match('(')) {
		term_action('(');
		next();
	}
	else{
		yyerror("'('");
		return 1;
	}
	if(expr_parse()){
		yyerror("expr");
		return 1;
	}
	if(match(')')) {
		term_action(')');
		next();
	}
	else{
		yyerror("')'");
		return 1;
	}
	term_action(1);
	return 0;

L_next_bin:
	next();
	if(match('(')) {
		term_action('(');
		next();
	}
	else{
		yyerror("'('");
		return 1;
	}
	if(expr_parse()){
		yyerror("expr");
		return 1;
	}
	if(match(',')) {
		term_action(',');
		next();
	}
	else{
		yyerror("','");
		return 1;
	}
	if(expr_parse()){
		yyerror("expr");
		return 1;
	}
	if(match(')')) {
		term_action(')');
		next();
	}
	else{
		yyerror("')'");
		return 1;
	}
	term_action(2);
	return 0;
}

int CParser::term_action(int token){
	int i,num;
	switch (token) {
		case ',':
		case ')':
		case '(':
			break;
		case DIV:
		case CNP:
		case ANP:
		case POW:
		case LOG:
		case LN:
		case LG:
		case EXP:
		case SIN:
		case COS:
		case TAN:
		case ABS:
		case ASIN:
		case ACOS:
		case ATAN:
		case NOT:
			if (++sp < MAX_DEPTH) result[sp] = token;
				else throw IDS_PARSER_STACK_OVF;
			break;
		case 1:
			sp--;
			switch ((int) result[sp]) {
				case LOG:
					result[sp] = log10(result[sp+1]);
					break;
				case LN:
					result[sp] = log(result[sp+1]);
					break;
				case LG:
					result[sp] = log(result[sp+1])/log(2);
					break;
				case EXP:
					result[sp] = exp(result[sp+1]);
					break;
				case SIN:
					result[sp] = sin(result[sp+1]);
					break;
				case COS:
					result[sp] = cos(result[sp+1]);
					break;
				case TAN:
					result[sp] = tan(result[sp+1]);
					break;
				case ABS:
					result[sp] = fabs(result[sp+1]);
					break;
				case ASIN:
					result[sp] = asin(result[sp+1]);
					break;
				case ACOS:
					result[sp] = acos(result[sp+1]);
					break;
				case ATAN:
					result[sp] = atan(result[sp+1]);
					break;
				case NOT:
					result[sp] = ! (int) (result[sp+1]);
					break;
			}
			break;
		case 2:
			sp -= 2;
			switch ((int) result[sp]) {
				case POW:
					result[sp] = pow(result[sp+1],result[sp+2]);
					break;
				case DIV:
					result[sp] = (int) result[sp+1] / (int) result[sp+2];
					break;
				case ANP:
					result[sp] = 1.0;i = 1;
					if (result[sp+1]<result[sp+2]) {
						result[sp] = -1.0;
						break;
					}
					for(num=(int) result[sp+1];i<=result[sp+2];num--,i++) 
						result[sp] = result[sp] * num; 
					break;
				case CNP:
					result[sp] = 1.0;i = 1;
					if (result[sp+1]<result[sp+2]) {
						result[sp] = -1.0;
						break;
					}
					for(num=(int) result[sp+1];num>result[sp+2];num--,i++) 
						result[sp] = result[sp] * num / i; 
					break;
			}
			break;
		case 0:
			break;
	}
	return 0;
}

int CParser::Zexpr_parse(void){
	if((!match(XOR))&&(!match(OR))&&(!match('-'))&&(!match('+'))&&(!match(','))&&(!match(')'))&&(!match(EOI))){
		yyerror("Zexpr");
	}
	else if(match(XOR)){
		Zexpr_action(XOR);
		goto L_next;
	}
	else if(match(OR)){
		Zexpr_action(OR);
		goto L_next;
	}
	else if(match('-')){
		Zexpr_action('-');
		goto L_next;
	}
	else if(match('+')){
		Zexpr_action('+');
		goto L_next;
	}
	else if(match(',')||match(')')||match(EOI)){
		return 0;
	}
	else return 1;
	Zexpr_action(0);
	return 0;

L_next:
	next();
	if(expr_parse()){
		yyerror("expr");
		return 1;
	}
	Zexpr_action(0);
	return 0;
}

int CParser::Zexpr_action(int token){
	if (token) {
		if (++sp < MAX_DEPTH) result[sp] = token;
			else throw IDS_PARSER_STACK_OVF;
	}
	else {
		/* combine the result of stmt and Zexpr */
		sp -= 2;
		switch ((int) result[sp+1]) {
			case '+':
				result[sp] = result[sp] + result[sp+2];
				break;
			case '-':
				result[sp] = result[sp] - result[sp+2];
				break;
			case OR:
				result[sp] = (int) result[sp] | (int) result[sp+2];
				break;
			case XOR:
				result[sp] = (int) result[sp] ^ (int) result[sp+2];
				break;
		}
	}
	return 0;
}

int CParser::stmt_parse(void){
	if((!match(ANP))&&(!match(POW))&&(!match(CNP))&&(!match(DIV))&&(!match(NOT))&&(!match(TAN))&&(!match(COS))&&(!match(SIN))&&(!match(ASIN))&&(!match(ACOS))&&(!match(ATAN))&&(!match(ABS))&&(!match(EXP))&&(!match(LG))&&(!match(LN))&&(!match(LOG))&&(!match('('))&&(!match(NAME))&&(!match('-'))&&(!match('+'))&&(!match(INTEGER))&&(!match(HEXVAL))){
		yyerror("stmt");
	}
	else if(match(ANP)||match(POW)||match(CNP)||match(DIV)||match(NOT)||match(TAN)||match(COS)||match(SIN)||match(ABS)||match(ASIN)||match(ACOS)||match(ATAN)||match(EXP)||match(LG)||match(LN)||match(LOG)||match('(')||match(NAME)||match('-')||match('+')||match(INTEGER)||match(HEXVAL)){
		if(term_parse()){
			yyerror("term");
			return 1;
		}
		if(Zstmt_parse()){
			yyerror("Zstmt");
			return 1;
		}
	}
	else return 1;
	stmt_action(0);
	return 0;
}

int CParser::stmt_action(int token){
	return 0;
}

int CParser::expr_parse(void){
	if((!match(ANP))&&(!match(POW))&&(!match(CNP))&&(!match(DIV))&&(!match(NOT))&&(!match(TAN))&&(!match(COS))&&(!match(ASIN))&&(!match(ACOS))&&(!match(ATAN))&&(!match(ABS))&&(!match(SIN))&&(!match(EXP))&&(!match(LG))&&(!match(LN))&&(!match(LOG))&&(!match('('))&&(!match(NAME))&&(!match('-'))&&(!match('+'))&&(!match(INTEGER))&&(!match(HEXVAL))){
		yyerror("expr");
	}
	else if(match(ANP)||match(POW)||match(CNP)||match(DIV)||match(NOT)||match(TAN)||match(COS)||match(SIN)||match(ABS)||match(ASIN)||match(ACOS)||match(ATAN)||match(EXP)||match(LG)||match(LN)||match(LOG)||match('(')||match(NAME)||match('-')||match('+')||match(INTEGER)||match(HEXVAL)){
		if(stmt_parse()){
			yyerror("stmt");
			return 1;
		}
		if(Zexpr_parse()){
			yyerror("Zexpr");
			return 1;
		}
	}
	else return 1;
	expr_action(0);
	return 0;
}

int CParser::expr_action(int token){
	return 0;
}

int CParser::yyparse(BOOL res){
	resolve = res;
	if((!match(ANP))&&(!match(POW))&&(!match(CNP))&&(!match(DIV))&&(!match(NOT))&&(!match(TAN))&&(!match(COS))&&(!match(ASIN))&&(!match(ACOS))&&(!match(ATAN))&&(!match(ABS))&&(!match(SIN))&&(!match(EXP))&&(!match(LG))&&(!match(LN))&&(!match(LOG))&&(!match('('))&&(!match(NAME))&&(!match('-'))&&(!match('+'))&&(!match(INTEGER))&&(!match(HEXVAL))&&(!match(EOI))){
		yyerror("macro");
	}
	else if(match(EOI)){
		return 2;
	}
	else if(match(ANP)||match(POW)||match(CNP)||match(DIV)||match(NOT)||match(TAN)||match(COS)||match(SIN)||match(ABS)||match(ASIN)||match(ACOS)||match(ATAN)||match(EXP)||match(LG)||match(LN)||match(LOG)||match('(')||match(NAME)||match('-')||match('+')||match(INTEGER)||match(HEXVAL)){
		if(expr_parse()){
			yyerror("expr");
			return 1;
		}
		if(match(EOI)) {
			yyparse_action(EOI);
			next();
		}
		else{
			yyerror("EOI");
			return 1;
		}
	}
	else return 1;
	yyparse_action(0);
	return 0;
}

int CParser::yyparse_action(int token){
	if (token) {
		sp--;
		result[sp] = result[sp+1];
	}
	return 0;
}

inline void CParser::yyerror(char *msg) {
	throw msg;
	return;
}

///////////////////////////////////////////////////////////////////////////
/// CMacro implementation

// Sets the macro's expression string
void CMacro::set_string(const char* buf) 
{	
	if (s_macro != NULL) {
		delete s_macro;
		s_macro = NULL;
	}
	if (buf != NULL && buf[0] != '\0') {
		s_macro = new char[LINE_LEN];
		strncpy(s_macro,buf,max(LINE_LEN,strlen(buf)));
	}
}

// Used to set the macro name if used in a global variable
void CMacro::set_name(const char *buf) 
{ 
	if (buf != NULL) 
		strncpy(name,buf,max(MAX_LEN,strlen(buf)));
	else name[0]='\0';
}

// Validates the expression's syntax.
int CMacro::validate(CCGMScriptDoc* pDoc)
{ 
	if (s_macro == NULL || s_macro[0] == '\0') 
		return 0;	// Fine
	CParser p(pDoc,s_macro);
	return p.yyparse(FALSE); 
}

// Evaluates the expression.
double CMacro::resolve(CCGMScriptDoc* pDoc) 
{ 
	double result = 0.0;
	if (!resolving) {
		CParser p(pDoc,s_macro);
		resolving = TRUE;		// Flip this macro's resolution semaphore ON
		if (!p.yyparse(TRUE))
			result = p.GetResult(); 
		else result = 0.0;
		resolving = FALSE;		// Flip this macro's resolution semaphore OFF
		return result;
	}
	else throw IDS_PARSER_REENTRANCE;
	return 0.0;
}

// Assignment operator.
CMacro& CMacro::operator=(const CMacro& elRHS) {
	resolving = elRHS.resolving;
	if (elRHS.s_macro) 
		set_string(elRHS.s_macro);
	set_name(elRHS.name);
	return *this;
}

void CMacro::Serialize(CArchive& ar) {
	// Instances of CMacro can be serialized by a direct call
	// to Serialize such as: m_Macro.Serialize(ar) or using the 
	// global operators defined below.
	if (ar.IsStoring())
	{	// storing code
		CString s_name(name);
		CString s_expr(s_macro);
		ar << s_name << s_expr;
	}
	else
	{	// loading code
		CString s_name;
		CString s_expr;
		resolving = FALSE;
		ar >> s_name >> s_expr;
		set_name(s_name);
		set_string(s_expr);
	}
}

// Global operator to define meaning of << for a CMacro type.
CArchive& operator<<(CArchive& ar, CMacro& m) {
	m.Serialize(ar);
	return ar;
}
// Global operator to define meaning of >> for a CMacro type.
CArchive& operator>>(CArchive& ar, CMacro& m) {
	m.Serialize(ar);
	return ar;
}
	
// Global DDX function for a CMacro Type.
void AFXAPI DDX_Macro(CDataExchange* pDX, int nIDC, CMacro& var)
{
	HWND hWndCtrl = pDX->PrepareEditCtrl(nIDC);
	char *s_macro = new char[LINE_LEN];
	ASSERT(hWndCtrl != NULL);
	ASSERT(s_macro != NULL);
	if	(pDX->m_bSaveAndValidate) {
		// Ctrl -> Variable
		::GetWindowText(hWndCtrl,s_macro,LINE_LEN);
		var.set_string(s_macro);
		try {
			var.validate(NULL);
		}
		catch (int id) {
			AfxMessageBox(id,MB_ICONSTOP | MB_OK);
			pDX->Fail();
		}
		catch (char *) {
			AfxMessageBox(IDS_SYNTAX_ERROR,MB_ICONSTOP | MB_OK);
			pDX->Fail();
		}
	}
	else {
		// Variable -> Ctrl
		::SetWindowText(hWndCtrl, var.get_string());
	}
	if (s_macro !=NULL) delete s_macro;
}

