//////////////////////////////////////////////////////////////////////////
/// Macros and Parser classes
/// 
#define EMPTY 1000
#define NO_TOKEN 1001
#define EOI 1003
#define NAME 1004
#define INTEGER 1005
#define HEXVAL 1006
#define OR 1007
#define XOR 1008
#define AND 1009
#define LOG 1010
#define LN 1011
#define LG 1012
#define EXP 1013
#define SIN 1014
#define COS 1015
#define TAN 1016
#define ABS 1017
#define ASIN 1018
#define ACOS 1019
#define ATAN 1020
#define NOT 1021
#define DIV 1022
#define CNP 1023
#define ANP 1024
#define POW 1025
#define macro 1026
#define expr 1027
#define stmt 1028
#define Zexpr 1029
#define term 1030
#define Zstmt 1031
#define number 1032
#define afloat 1033
#define decimal 1034
#define exponent 1035
#define Eexp 1036

/* Max depth of a parse tree (stack size) */
#define MAX_DEPTH 	32
/* Max useful length of an identifier */
#define MAX_LEN 	12
/* Max input line length */
#define LINE_LEN 	80
/* Invalid value */
#define INVALID_MACRO	atof("hgfhgftr")


class CCGMScriptDoc;

////////////////////////////////////////////////////////////////////////////
// Parser instances are created by Macro instances for the purpose
// of evaluating their own value. The parser queries the View for global 
// macros (not within elements) if necessary.

class CParser
{
public:
	// Construction/Destruction
	CParser(CCGMScriptDoc*, const char*);
	~CParser() { delete(inbuf); }
	int yyparse(BOOL);
	double GetResult(void) { return result[0]; }

private:
	char *p;			/* token pointer */
	char *yytext;		/* pointer to character pattern of token */
	char *yylast;		/* pointer to last token */
	int yyleng;			/* length of current token */
	int lookahead;		/* look-ahead token */
	char *inbuf;		/* input buffer */
	BOOL resolve;		/* whether to resolve macros */
	int sp;						/* stack pointer */
	double result[MAX_DEPTH];	/* Return values stack */
	CCGMScriptDoc* pDoc;		/* Document pointer */


	int match(int);
	void next(void);
	void yyerror(char *);
	int yylex(void);

	int Eexp_parse(void);
	int Eexp_action(int token);
	int exponent_parse(void);
	int exponent_action(int token);
	int decimal_parse(void);
	int decimal_action(int token);
	int afloat_parse(void);
	int afloat_action(int token);
	int number_parse(void);
	int number_action(int token);
	int Zstmt_parse(void);
	int Zstmt_action(int token);
	int term_parse(void);
	int term_action(int token);
	int Zexpr_parse(void);
	int Zexpr_action(int token);
	int stmt_parse(void);
	int stmt_action(int token);
	int expr_parse(void);
	int expr_action(int token);
	int yyparse_action(int token);

};

class CMacro : public CObject
{
public:
	CMacro() { s_macro = NULL;name[0]='\0';resolving=FALSE; }
	~CMacro() { if (s_macro) delete s_macro; }
	void set_string(const char *);
	void set_name(const char *);
	const char* get_string() { return s_macro; }
	const char* get_name() { return name; }
	CMacro& operator=(const CMacro&);
	friend CArchive& operator<<(CArchive& ar, CMacro& m);
	friend CArchive& operator>>(CArchive& ar, CMacro& m);
	void Serialize(CArchive& ar);

	int validate(CCGMScriptDoc* pDoc);
	double resolve(CCGMScriptDoc* pDoc);
	BOOL isNull() { return (s_macro == NULL); }

private:
	BOOL	resolving;	// Semaphore to avoid re-entrance
	char *s_macro;		// String value or macro name
	char name[MAX_LEN];
};

// Global DDX entry for Macro Parsing
void AFXAPI DDX_Macro(CDataExchange* pDX, int nIDC, CMacro& dfMacro);
	