// CEDatabase.cpp : Defines the class behaviors for the CEDatabase classes
// Copyright (C) 1998, John C. Schettino Jr.

#include "stdafx.h"

#include "CEDatabase.h"
#include <tchar.h>


//////////////////////////////////////////////////////////////////////
// CCeFindDBDatabase Class
//////////////////////////////////////////////////////////////////////


CCeFindDBDatabaseIterater *
CCeFindDBDatabase::Find(CEQuery & findSpec)
{
	if (m_hHandle == INVALID_HANDLE_VALUE)
		throw CEDatabaseException(CEDatabaseException::Err_Deleted);

	CCeDBQueryProp * keyField = FindKey(findSpec);
	CCeFindDBDatabaseIterater * new_search = 
		new CCeFindDBDatabaseIterater(&findSpec, keyField, this);
	return new_search;
}

bool
CCeFindDBDatabase::TestFind(CEQuery & findSpec)
{
	if (m_hHandle == INVALID_HANDLE_VALUE)
		throw CEDatabaseException(CEDatabaseException::Err_Deleted);

	CCeDBProp * keyField = FindKey(findSpec);
	return (!keyField);
}

CCeDBQueryProp *
CCeFindDBDatabase::FindKey(CEQuery & findSpec)
{
	// search findSpec for a match on the index
	// return highest weighted query field that matches index
	// weight == higher than >, <
    // unless CEQuery specifies a sort order

	WORD order;
	CCeDBQueryProp * sortField = findSpec.sortField();

	if (sortField)
	{
		if (!IsIndexed(sortField->GetIdent(), order))
			throw CEDatabaseException(CEDatabaseException::Err_Invalid_Sort);
	}
	else
	{
		// select highest weighted indexed query field
		int i;

		int *weights = new int[(findSpec.numFields()+1)];
		memset(weights, 0, sizeof(int) * (findSpec.numFields()+1));

		CCeDBQueryProp * keyField = 0;
		for (keyField = findSpec[i=0]; keyField; keyField = findSpec[++i])
		{
			if (IsIndexed(keyField->GetIdent(), order)) // only weigh indexed fields
			{
				switch (keyField->getOperation())
				{
				case CCeDBQueryProp::Equal:	// higher weight for fields with an == test
					weights[i] +=10;
					break;
				case CCeDBQueryProp::LE:
					if (order == CCeDBProp::Sort_Descending)
						weights[i] += 5;
					break;
				case CCeDBQueryProp::GE:
					if (order == CCeDBProp::Sort_Ascending)
						weights[i] += 5;
					break;
				}
			}
		}
		weights[i] = -1;

		// select field with highest weight
		int maxWeight = 0;
		for (i=0; weights[i] > 0; i++)
			if (weights[i] > weights[maxWeight]) 
				maxWeight = i;
		
		if (weights[maxWeight] > 0) 
			sortField = findSpec[maxWeight];
		delete[] weights;
	}
	
	return sortField;
}

bool
CCeFindDBDatabase::IsIndexed(CEPROPID ident, WORD &order)
{
	CCeDBProp sortProps[CEDB_MAXSORTORDER];
	int num_props;

	GetSortProps(&num_props, sortProps);
	if (num_props == 0) return false;

	for (int i = 0; i < num_props; i++)
		if (ident == sortProps[i].GetIdent())
		{
			order = sortProps[i].GetSortFlags();
			return true;
		}

	return false;
}

//////////////////////////////////////////////////////////////////////
// CCeFindDBDatabase Class
//////////////////////////////////////////////////////////////////////
CCeFindDBDatabaseIterater::CCeFindDBDatabaseIterater(
	CEQuery *q, 
	CCeDBQueryProp * key, 
	CCeDBDatabase * db)
{
	// allocate copies of all parms
	CCeDBQueryProp *_myKey = (key) ? new CCeDBQueryProp(*key) : 0;
	CCeDBProp openKey;

	if (_myKey)
	{
		CCeDBProp sortProps[CEDB_MAXSORTORDER];
		int num_props;

		CEPROPID ident = _myKey->GetIdent();
		db->GetSortProps(&num_props, sortProps);
		for (int i = 0; i < num_props; i++)
			if (ident == sortProps[i].GetIdent())
			{
				_keyOrder = sortProps[i].GetSortFlags();
				openKey = sortProps[i];
				break;
			}
	}
	else
		_keyOrder = CCeDBProp::Sort_GenericOrder;


	_myDb = new CCeDBDatabase();
	_myDb->m_bAutoSeekNext = false;
	_myDb->Open( db->m_CEOID, _myKey ? &openKey : 0); // open using selected key field
	_myQuery = new CEQuery(*q);

	// do a sequential search if no key field
	// or if no search criteria exist on the key field
	_primaryQuery = (key==0) ? 0 : _myQuery->testOn(key->GetIdent(), _keyOrder);
	_start = true;
	_eof = false;
	_hits = _tests = 0;
}

CCeFindDBDatabaseIterater::~CCeFindDBDatabaseIterater(void)
{
}

CCeFindDBDatabaseIterater& 
CCeFindDBDatabaseIterater::operator++()
{
	next();
	return *this;
}

CCeFindDBDatabaseIterater 
CCeFindDBDatabaseIterater::operator++(int)
{
	CCeFindDBDatabaseIterater temp = *this;
	next();
	return temp;
}
	

CCeDBRecord* 
CCeFindDBDatabaseIterater::current()
{
	CCeDBRecord * _rc = new CCeDBRecord();
	if (!_myDb->ReadCurrRecord(_rc))
	{
		if (_rc) delete _rc;
		_rc = 0;
	}

	return _rc;
}

CCeDBRecord*
CCeFindDBDatabaseIterater::first()
{
	_start = true;
	_hits = _tests = 0;
	return next();
}

CCeDBRecord*
CCeFindDBDatabaseIterater::next()
{
	if (_eof) return 0;
	CEOID found_oid;
	int grace = 1; // work around

	if (_start)
	{
		// Locate first value in index (if possible)
		if (!_primaryQuery || 
			((_keyOrder == CCeDBProp::Sort_Ascending) && 
				(_primaryQuery->getOperation() == CCeDBQueryProp::LE)) ||
			((_keyOrder == CCeDBProp::Sort_Descending) &&
				(_primaryQuery->getOperation() == CCeDBQueryProp::GE)))
			found_oid = _myDb->SeekFirst();
		else
		{
			if (_primaryQuery->getOperation() == CCeDBQueryProp::Equal) 
				found_oid =_myDb->SeekFirstEqual(*_primaryQuery);
			else if (_primaryQuery->getOperation() == CCeDBQueryProp::GE)
				found_oid = _myDb->SeekValueGreater(*_primaryQuery);
			else
			{
				found_oid = _myDb->SeekValueSmaller(*_primaryQuery);
			}
		}
		_start = false;
	}
	else
	{
		if (!_primaryQuery || 
			(_primaryQuery->getOperation() != CCeDBQueryProp::Equal)) 
			found_oid = _myDb->SeekNext();
		else 
			found_oid = _myDb->SeekNextEqual(*_primaryQuery);
	}

	// get record (if any)
	CCeDBRecord* r = 0;
	if (found_oid == 0)
		_eof = true;
	else
	{
		r = current();
		while (r && !_myQuery->match(r, _eof, grace) && !_eof && !_myDb->m_bEOF)
		{
			_tests++;
			if (!_primaryQuery || 
				(_primaryQuery->getOperation() != CCeDBQueryProp::Equal)) 
				found_oid = _myDb->SeekNext();
			else 
				found_oid = _myDb->SeekNextEqual(*_primaryQuery);
			if (r) delete r;
			r = (found_oid != 0) ? current() : 0;
		}

		if (!r || _eof || _myDb->m_bEOF) 
		{
			if (r) delete r;
			r = 0;
			_eof = true;
		}
		else 
			_hits++;
	}
	return r;
}

void 
CCeFindDBDatabaseIterater::stats(int &hits, int &tests, bool &range)
{
	hits = _hits;
	tests = _tests;
	range = _myQuery->_range;
}




//////////////////////////////////////////////////////////////////////
// CEDatabaseException Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEDatabaseException::CEDatabaseException(errors e)
{
	_ceError = e;
	_errorCode = GetLastError();
}

CEDatabaseException::errors
CEDatabaseException::getCeError()
{
	return _ceError;
}

DWORD
CEDatabaseException::getWinError()
{
	return _errorCode;
}

CEDatabaseException::~CEDatabaseException()
{
;
}


//////////////////////////////////////////////////////////////////////
// CCeDBQueryProp Class
//////////////////////////////////////////////////////////////////////

void
CCeDBQueryProp::setOperation(FieldOperation op)
{
	_operation = op;
}

CCeDBQueryProp::FieldOperation 
CCeDBQueryProp::getOperation()
{
	return _operation;
}

//////////////////////////////////////////////////////////////////////
// CEQuery Class
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CEQuery::CEQuery()
{
	_numFields = 0;
	_maxFields = CEDB_MAXSORTORDER;
	_fields = new CCeDBQueryProp *[CEDB_MAXSORTORDER];
	_sortField = 0;
	_range = false;

}

CEQuery::~CEQuery()
{
	if (_sortField) delete _sortField;

	for (int i=0; i < _numFields; i++)
		delete(_fields[i]);

	delete[] _fields;
}

void
CEQuery::addField(CCeDBQueryProp * f)
{
	if (_numFields == _maxFields)
	{
		_maxFields += CEDB_MAXSORTORDER;
		CCeDBQueryProp ** temp = new CCeDBQueryProp *[_maxFields];

		for (int i = 0; i < _numFields; i++)
			temp[i] = _fields[i];

		delete[] _fields;
		_fields = temp;
	}

	_fields[_numFields++] = f;
}

void
CEQuery::setSortField(CCeDBQueryProp *f)
{
	_sortField = f;
}

int 
CEQuery::numFields(void)
{
	return _numFields;
}

CCeDBQueryProp *
CEQuery::sortField()
{
	return _sortField;
}

CCeDBQueryProp * 
CEQuery::operator[](int fieldNo)
{
	return (fieldNo < _numFields) ? _fields[fieldNo] : 0;
}

CCeDBQueryProp *
CEQuery::testOn(WORD id, WORD order)
{
	// find best field to do index test on
	// assumes "and" search only

	// any field matching id and using eq test wins
	// next best is greater or equal
	// worst is le

	CCeDBQueryProp * fallback = 0;
	bool _ge = false, _le = false;
	_index = id;
	_range = false;

	for (int i=0; i < _numFields; i++)
		if (_fields[i]->GetIdent() == id)
		{
			switch (_fields[i]->getOperation())
			{
			case CCeDBQueryProp::Equal:
				return _fields[i];
			case CCeDBQueryProp::GE:
				_ge = true;
				if (order == CCeDBProp::Sort_Ascending)
					fallback = _fields[i];
				else
				{
					if (!fallback) fallback = _fields[i];
				}
				break;
			case CCeDBQueryProp::LE:
				_le = true;
				if (order == CCeDBProp::Sort_Descending)
					fallback = _fields[i];
				else
				{
					if (!fallback) fallback = _fields[i];
				}
				break;
			}
		}
	_range = (_ge && _le);
	return fallback;
}

bool
CEQuery::match(CCeDBRecord* r, bool &eof, int &grace)
{
	CCeDBProp* p;
	WORD id;
	WORD range_id = -1;

	// does current record match?
	// more will be false if there are no more possible matches

	if (_range)
	{
		// check index field first
		// if it false then no match *and* no more
		range_id = _index;
		p =  r->GetPropFromIdent(range_id);

		// find all search critera using the range property
		for (int i=0; i < _numFields; i++)
			if (_fields[i]->GetIdent() == range_id)
				if (!matchField(p, i))
				{
					// no more matches possible
					// allow a 1 record overlap for 
					// SEEKVALUESMALLER bug
					eof = (grace-- == 0);
					return false;
				}
	}

	// still here? either no range or
	// range and index field within range

	// foreach property in r, perform all tests
	// skip _index if we already tested it in range test
	int maxProp = r->GetNumProps();
	for (int i=0; i < maxProp; i++)
	{
		// get the property
		p =  r->GetPropFromIndex(i);
		id = p->GetIdent();

		if (_range && (range_id == id)) 
			continue; // skip range field if appropriate

		// find all search critera using this property
		for (int i=0; i < _numFields; i++)
			if ((_fields[i]->GetIdent() == id) && 
				(!matchField(p, i)))
				return false; // assuming "AND" operator between each test

	}
	return true; // passed all tests.
}

bool
CEQuery::matchField(CCeDBProp* p, int qField)
{
	int comp_val = 0;
	switch (p->GetType())
	{
	case CCeDBProp::Type_Short: // A 16-bit signed integer. 
		comp_val = (p->GetShort() - _fields[qField]->GetShort());
		break;
	case CCeDBProp::Type_UShort: // A 16-bit unsigned integer. 
		comp_val = (p->GetUShort() - _fields[qField]->GetUShort());
		break;


	case CCeDBProp::Type_Long: // A 32-bit signed integer. 
		comp_val = (p->GetLong() - _fields[qField]->GetLong());
		break;

	case CCeDBProp::Type_ULong: //	A 32-bit unsigned integer. 
		comp_val = (p->GetULong() - _fields[qField]->GetULong());
		break;

	case CCeDBProp::Type_Filetime: // A FILETIMEstructure. 
		comp_val = CompareFileTime(&p->GetFiletime(), &_fields[qField]->GetFiletime());
		break;


	case CCeDBProp::Type_String: // A null-terminated string. Unicode assumed.
		comp_val = wcscmp(p->GetString(), _fields[qField]->GetString());
		break;

	//case Type_Blob:	// not supported! A CEBLOBstructure. 	
	//	break;
	}

	// now check for desired result
	switch (_fields[qField]->getOperation())
	{
	case CCeDBQueryProp::Equal:
		return (comp_val == 0);
	case CCeDBQueryProp::GE:
		return (comp_val >= 0);
	case CCeDBQueryProp::LE:
		return (comp_val <= 0);
	}

	// NOT REACHED
	return false;
}
