// CEDatabase.h : header file for the CEDATABASE classes
// Copyright (C) 1998, John C. Schettino Jr.

#include <Wcedb.h>			// CE2.0 database classes

class CString;
class CEIndex;
class CEQuery;
class CCeDBQueryProp;

// iterater (and db search) can only go forward

class CCeFindDBDatabaseIterater
{
public:
	CCeFindDBDatabaseIterater(CEQuery *, CCeDBQueryProp *, CCeDBDatabase *);
	~CCeFindDBDatabaseIterater();

	CCeFindDBDatabaseIterater& operator++();       // Prefix increment operator.
	CCeFindDBDatabaseIterater operator++(int);     // Postfix increment operator
	
	CCeDBRecord * operator()() {return current();}	// current item accessor
	CCeDBRecord * current();

	CCeDBRecord * first();		// resets search
	CCeDBRecord * next();

	void stats(int &, int &, bool &);

private:
	bool _start;
	bool _eof;
	int _hits;
	int _tests;

	CCeDBQueryProp * _primaryQuery;
	CCeDBDatabase * _myDb;	// local copy of db
	CEQuery * _myQuery;		// local copy of query spec
	WORD _keyOrder;
};


class CCeFindDBDatabase  : public CCeDBDatabase
{
public:
	CCeFindDBDatabaseIterater * Find (CEQuery & );
	bool TestFind (CEQuery & );

private:
	CCeDBQueryProp * FindKey(CEQuery &);
	bool IsIndexed(CEPROPID, WORD &);

};

// CEDatabase exception class, thrown by all CEDatabase classes on error

class CEDatabaseException  
{
public:
	enum errors {
		Err_Open,
		Err_Create,
		Err_Deleted,
		Err_Delete,
		Err_Invalid_Sort,
		Err_Get_Info,
		Err_Set_Info,
		Err_Add_Record,
		Err_Update_Invalid,
		Err_Update_Record,
		Err_Delete_Invalid,
		Err_Delete_Record,
		Err_MaxSortOrder
	};

	CEDatabaseException(errors);
	virtual ~CEDatabaseException();

	errors getCeError();
	DWORD getWinError();

private:
	errors _ceError;  // the CEDatabase error
	DWORD _errorCode; // the Win32 error code from GetLastError()
};

class  CCeDBQueryProp : public CCeDBProp
{
public:

	enum FieldOperation {
		Equal,
		LE,
		GE
	};

	void setOperation(FieldOperation);
	FieldOperation getOperation();

private:
	FieldOperation _operation;
};

class CEQuery
{
	friend class CCeFindDBDatabaseIterater;
public:
	CEQuery();
	virtual ~CEQuery();

	void addField(CCeDBQueryProp *);
	void setSortField(CCeDBQueryProp *);
	int numFields(void);

	CCeDBQueryProp * testOn(WORD, WORD);
	bool match(CCeDBRecord*, bool &, int &);

	CCeDBQueryProp * sortField();
	CCeDBQueryProp * operator[](int fieldNo);

private:
	bool matchField(CCeDBProp*, int);

	CCeDBQueryProp ** _fields;
	CCeDBQueryProp * _sortField;

	int _numFields;
	int _maxFields;

	WORD _index;
	bool _range;
};
