/* @#START#@
**
** Copyright:
**	(C) Copyright International Business Machines Corporation 1996
**	All Rights Reserved
**	Licensed Materials - Property of IBM
**	US Government Users Restricted Rights - Use, duplication, or disclosure
**	restricted by GSA ADP Schedule Contract with IBM Corp.               
**
** Module:
**
** Description:
**	Memory allocation for FMA. BSD4.4 memory allocation techniques
**	for performance analysis.
**
** Functions:
**
** Dependencies:
**
** Context:
**
** Restrictions:
**
** Environment:
**	AIX 4.1.4 with CSET++ 3.1.4
**
** Linkage:
**
** Entry points:
**
** Input:
**
** Output:
**
** Return:
**
** External references:
**
** History:
** Date       Author  Description
** -----------------------------------------------------------------------------
** 12-Jul-97  Richter Created
**
** @#END#@*/

#include	<unistd.h>

#include	"fmaheap.h"

extern struct t_heapmgr *heapmgr_ptr;

void *fmaalloc(size_t len)
{
    size_t roundoff = 0;
    struct t_memchunk *here, *outptr;
    int rc;

    /* Lock it if shared memory pool */
    if( heapmgr_ptr->shared >= 1 && (rc = fmalock(heapmgr_ptr)) )
	return 0;

    for( ; len > 1 << roundoff + MINSIZE_2; ++roundoff )
	;
    if( roundoff == PAGESIZE_2 - MINSIZE_2 ){	/* Allocate full page */
	outptr = getpage(roundoff);
	goto out;
    }
    if( here = heapmgr_ptr->freelist[roundoff] ){
	heapmgr_ptr->freelist[roundoff] = here->page;
	outptr = here;
    }else{
	here = getpage(roundoff);
	heapmgr_ptr->freelist[roundoff] = here;
	roundoff += MINSIZE_2;
	outptr = sliceit(here, 1 << roundoff, (1 << PAGESIZE_2 - roundoff) - 1);
    }
out:
    if( heapmgr_ptr->shared >= 1 && (rc = fmaunlock(heapmgr_ptr)) )
	return 0;
    return outptr;
}

/*
** Return a page to the page table. Check heuristics:
** - Last page in use returned, reset lastused to zero.
** - Last page of used range retunred, decrement lastused counter.
** This function must be called in locked state if sharing possible.
*/
static void putpage(unsigned long map_idx, union t_pageentry *pep)
{
    /*
    ** If the last page in used is returned, we reset the lastused
    ** and nofree counter to zero.
    */
    if( heapmgr_ptr->nofree + 1 == heapmgr_ptr->lastused ){
	heapmgr_ptr->nofree = 0;
	heapmgr_ptr->lastused = 0;
	heapmgr_ptr->anchor = ENDOFLIST;
	return;
    }
    /*
    ** If this is the last used page we just free'd, decrement
    ** last_used counter, otherwise insert into freelist.
    */
    if( map_idx == heapmgr_ptr->lastused - 1 ){
	--heapmgr_ptr->lastused;
	return;
    }
    ++heapmgr_ptr->nofree;
    pep->freelist.succ = heapmgr_ptr->anchor;
    pep->freelist.size = 1;
    heapmgr_ptr->anchor = map_idx;
}

void fmafree(void *addr)
{
    unsigned long map_idx = (addr - heapmgr_ptr->heapaddr) >> PAGESIZE_2;
    struct t_memchunk *chunk = (struct t_memchunk *)addr;
    union t_pageentry *pep = &heapmgr_ptr->ptab[map_idx];
    int slot = pep->pgaddr.slot, rc;

    /* Lock it if shared memory pool */
    if( heapmgr_ptr->shared >= 1 && (rc = fmalock(heapmgr_ptr)) )
	return;

    if( slot ==  PAGESIZE_2 - MINSIZE_2 ){	/* Full page returned */
	putpage(map_idx, pep);
	goto out;
    }
    chunk->page = heapmgr_ptr->freelist[slot];
    heapmgr_ptr->freelist[slot] = chunk;
out:
    if( heapmgr_ptr->shared >= 1 )
        fmaunlock(heapmgr_ptr);
}
