/* @#START#@
**
** Copyright:
**	(C) Copyright International Business Machines Corporation 1996
**	All Rights Reserved
**	Licensed Materials - Property of IBM
**	US Government Users Restricted Rights - Use, duplication, or disclosure
**	restricted by GSA ADP Schedule Contract with IBM Corp.               
**
** Module:
**	fmalloc.c
**
** Description:
**	Memory heap management FMA
**
** Functions:
**
** Dependencies:
**
** Context:
**
** Restrictions:
**
** Environment:
**	AIX 4.1.4 with CSET++ 3.1.4
**
** Linkage:
**
** Entry points:
**
** Input:
**
** Output:
**
** Return:
**
** External references:
**
** History:
** Date       Author  Description
** -----------------------------------------------------------------------------
** 22-Jan-97  Richter Created
**
** @#END#@*/

#include	<unistd.h>

#include	"fmaheap.h"

extern struct t_heapmgr *heapmgr_ptr;

void *fmaalloc(size_t len)
{
    size_t roundoff = 0;
    struct t_memchunk *here, *outptr;
    int rc;

    /* Lock it if shared memory pool */
    if( heapmgr_ptr->shared >= 1 && (rc = fmalock(heapmgr_ptr)) )
	return 0;

    for( ; len > 1 << roundoff + MINSIZE_2; ++roundoff )
	;
    if( roundoff == PAGESIZE_2 - MINSIZE_2 ){	/* Allocate full page */
	outptr = getpage(roundoff);
	goto out;
    }
    if( here = heapmgr_ptr->freelist[roundoff] ){
	if( --here->cnt == 0 ){
	    unsigned long map_idx = ((void *)here - heapmgr_ptr->heapaddr) >> PAGESIZE_2;
	    heapmgr_ptr->ptab[map_idx].pgaddr.addr = 0;
	    if( heapmgr_ptr->freelist[roundoff] = here->succ )
		heapmgr_ptr->freelist[roundoff]->pred = 0;
	    outptr = here;
	}else{
	    struct t_memchunk *tmp = here->page;
	    here->page = tmp->page;
	    outptr = tmp;
	}
    }else{
	here = getpage(roundoff);
        here->pred = here->succ = NULL;
	heapmgr_ptr->freelist[roundoff] = here;
	roundoff += MINSIZE_2;
	here->cnt = (1 << (PAGESIZE_2 - roundoff)) - 1;
	outptr = sliceit(here, 1 << roundoff, here->cnt);
    }
out:
    if( heapmgr_ptr->shared >= 1 && (rc = fmaunlock(heapmgr_ptr)) )
	return 0;
    return outptr;
}

/*
** Return a page to the page table. Check heuristics:
** - Last page in use returned, reset lastused to zero.
** - Last page of used range retunred, decrement lastused counter.
** This function must be called in locked state if sharing possible.
*/
static void putpage(unsigned long map_idx, union t_pageentry *pep)
{
    /*
    ** If the last page in used is returned, we reset the lastused
    ** and nofree counter to zero.
    */
    if( heapmgr_ptr->nofree + 1 == heapmgr_ptr->lastused ){
	heapmgr_ptr->nofree = 0;
	heapmgr_ptr->lastused = 0;
	heapmgr_ptr->anchor = ENDOFLIST;
	return;
    }
    /*
    ** If this is the last used page we just free'd, decrement
    ** last_used counter, otherwise insert into freelist.
    */
    if( map_idx == heapmgr_ptr->lastused - 1 ){
	--heapmgr_ptr->lastused;
	return;
    }
    ++heapmgr_ptr->nofree;
    pep->freelist.succ = heapmgr_ptr->anchor;
    pep->freelist.size = 1;
    heapmgr_ptr->anchor = map_idx;
}

void fmafree(void *addr)
{
    unsigned long map_idx = (addr - heapmgr_ptr->heapaddr) >> PAGESIZE_2;
    struct t_memchunk *append;
    struct t_memchunk *chunk = (struct t_memchunk *)addr;
    union t_pageentry *pep = &heapmgr_ptr->ptab[map_idx];
    int slot = pep->pgaddr.slot, rc;

    /* Lock it if shared memory pool */
    if( heapmgr_ptr->shared >= 1 && (rc = fmalock(heapmgr_ptr)) )
	return;

    if( slot ==  PAGESIZE_2 - MINSIZE_2 ){	/* Full page returned */
	putpage(map_idx, pep);
	goto out;
    }
    if( append = (struct t_memchunk *)(pep->pgaddr.addr << MINSIZE_2) ){
	chunk->page = append->page;
	append->page = chunk;
	if( ++append->cnt == 1 <<  PAGESIZE_2 - MINSIZE_2 - slot ){
	    if( append->succ )
		append->succ->pred = append->pred;
	    if( append->pred )
		append->pred->succ = append->succ;
	    else
		/* No predessor, new head of freelist */
		heapmgr_ptr->freelist[slot] = append->succ;
	    putpage(map_idx, pep);
	}
    }else{	/* First free chunk of this page */
	pep->pgaddr.addr = (unsigned long)chunk >> MINSIZE_2;
	chunk->page = NULL;
	chunk->cnt = 1;
	if( heapmgr_ptr->freelist[slot] == NULL ){
	    /* First freelist entry of this size */
	    chunk->succ = chunk->pred = NULL;
	    heapmgr_ptr->freelist[slot] = chunk;
	}else{
	    /* Append into freelist of pages with this size */
	    register struct t_memchunk *head = heapmgr_ptr->freelist[slot];
	    if( chunk->succ = head->succ )
		head->succ->pred = chunk;
	    head->succ = chunk;
	    chunk->pred = head;
	}
    }
out:
    if( heapmgr_ptr->shared >= 1 )
        fmaunlock(heapmgr_ptr);
}
