/* @#START#@
**
** Copyright:
**	(C) Copyright International Business Machines Corporation 1996
**	All Rights Reserved
**	Licensed Materials - Property of IBM
**	US Government Users Restricted Rights - Use, duplication, or disclosure
**	restricted by GSA ADP Schedule Contract with IBM Corp.               
**
** Module:
**	fmaheap.h
**
** Description:
**
** Functions:
**
** Dependencies:
**
** Context:
**
** Restrictions:
**
** Environment:
**	AIX 4.1.4 with CSET++ 3.1.4
**
** Version:
**	%W% %I% %D% %T%
**
** History:
** Date       Author  Description
** -----------------------------------------------------------------------------
** 19-Nov-96  Richter Created
**
** @#END#@*/

#ifndef fmaalloc_h
#define	fmaalloc_h

#include	<sys/types.h>
#include	<sys/mode.h>
#include	<sys/ipc.h>
#include	<sys/shm.h>
#include	<sys/sem.h>

#include	"fma.h"		/* Include user defines */

#define	PAGESIZE_2	12	/* Page size exponent with base 2 */
#define	MINSIZE_2	4	/* Smallest allocatable unit 2 ^ MINSIZE_2) */
#define	MAXSIZE_2 	20	/* Largest allocatable unit 2 ^ MAXSIZE_2) */
#ifdef PAGESIZE
#   if PAGESIZE != (1 << PAGESIZE_2)
#      error define PAGESIZE different from Operating system define
#   endif
#else
#define	PAGESIZE	(1 << PAGESIZE_2)	/* Size of a page in bytes */
#endif
#define	FLSIZE		(PAGESIZE_2 - MINSIZE_2)	/* Size of freelist */
#define	PTABSIZE	65	/* Maximum size of page table in pages */
#define	MAXMEMORY	((1 << 28)/(1 << PAGESIZE_2) - PTABSIZE)
				/* Maximum allocatable memory in pages */

/*
** Macro for allocation of memory. Round up to the next power of 2 element
*/
#define	MINSIZE_BM	0xf	/* Bitmap MINSIZE roundup */

/* Get base part of an address */
#define	PAGE_BM		0xfffff000
#define GETPAGE(x)	((unsigned long)(x) & PAGE_BM)

/*
** Round bytes into number of pages needed to store bytes.
** Round bytes into number of pages and back into bytes.
*/
#define	NUMPAGE(x)	(((x) + PAGESIZE - 1) >> PAGESIZE_2)
#define	FITPAGE(x)	(NUMPAGE(x) << PAGESIZE_2)

struct t_memchunk {		/* Chunk of memory on any freelist */
    struct t_memchunk *succ,	/* Successor on free list, next page */
		    *pred,	/* Predessor on free list, next page */
		    *page;	/* List of free chunks on this page */
    unsigned short cnt;		/* Counter, used in first entry in list */
};

union t_pageentry {		/* Page entry, one for each page */
    struct {			/* Double linked free list of pages */
	unsigned short size,	/* Size in page table entries */
			succ;	/* Successor entry in free page list */
    } freelist;
    struct {			/* Address decoding */
	int slot:4;		/* Size of memory chunks on this page */
	int addr:28;		/* Address of memory chunk */
    } pgaddr;			/* Page address for split up */
    struct t_memchunk *addr;	/* First memory address */
};

struct t_heapmgr {		/* Heap Management */
    void *baseaddr;		/* Base address of heap administration */
    void *heapaddr;		/* Base address of heap memory */
    unsigned long lastused;	/* # of last used page */
    unsigned long maxused;	/* # of last used page for statistics */
    unsigned long endaddr;	/* Address of last accessible page */
    unsigned long nofree;	/* # of free pages in list */
    int shared;			/* True if shared between processes */
    int	pagesize;		/* Pagesize as retrieved from OS */
    unsigned short ptabsz;	/* Size of page table in entries */
    size_t admin;		/* # of bytes allocated for administration */
    size_t length;		/* # of bytes allocated for memory */
    key_t shmid;		/* ID of shared memory */
    key_t semid;		/* ID of semaphore */
    unsigned short anchor;	/* First free element in page table */
    struct t_memchunk *freelist[PAGESIZE_2 - MINSIZE_2];
    union t_pageentry ptab[1];	/* Page table */
};

#define	ENDOFLIST	0xffff

/*
** Prototypes
*/
void *getpage(size_t);
void *sliceit(struct t_memchunk *, size_t, unsigned short);

#endif
