/* @#START#@
**
** Copyright:
**	(C) Copyright International Business Machines Corporation 1996
**	All Rights Reserved
**	Licensed Materials - Property of IBM
**	US Government Users Restricted Rights - Use, duplication, or disclosure
**	restricted by GSA ADP Schedule Contract with IBM Corp.               
**
** Module:
**	fmainit.c
**
** Description:
**	Initialize Memory region for FMA
**
** Functions:
**
** Dependencies:
**
** Context:
**
** Restrictions:
**
** Environment:
**	AIX 4.1.2 with CSET++ 3.1.3
**
** Linkage:
**
** Entry points:
**
** Input:
**
** Output:
**
** Return:
**
** External references:
**
** Version:
**	%W% %I% %D% %T%
**
** History:
** Date       Author  Description
** -----------------------------------------------------------------------------
** 19-Nov-96  Richter Created
**
** @#END#@*/

#include	<stdio.h>
#include	<stdlib.h>
#include	<errno.h>

#include	"fmaheap.h"

#define	KEYFILE	"/home/richter"

struct t_heapmgr *heapmgr_ptr;

/*
** Prepare the memory management for being shared by several processes.
** Create semaphore for access protection.
*/
static int mkshared(struct t_heapmgr *heapp, size_t len, char keychar)
{
    key_t key = ftok(KEYFILE, keychar);
    int mode = S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH | S_IRUSR | S_IWUSR |
		IPC_CREAT | IPC_EXCL;

    if( (heapp->shmid = shmget(key, len, mode)) ==  -1 )
	return errno;
    if( (heapp->baseaddr = shmat(heapp->shmid, 0, 0)) == (void *)-1 ){
	shmctl(heapp->shmid, IPC_RMID, 0);
	return -3;
    }
    /* Create private sema4 for shared memory access protection */
    key = IPC_PRIVATE;
    if( (heapp->semid = semget(key, 1, mode)) == -1 ){
	int error = errno;
	shmdt(heapp->baseaddr);
	shmctl(heapp->shmid, IPC_RMID, 0);
	return error;
    }
    /* Set initial value to 1 */
    if( semctl(heapp->semid, 0, SETVAL, 1) == -1 ){
	int error = errno;
	semctl(heapp->semid, 0, IPC_RMID);
	shmdt(heapp->baseaddr);
	shmctl(heapp->shmid, IPC_RMID, 0);
	return error;
    }
    heapp->shared = 1;
    return 0;
}

/*
** Memory management segment has been prepared for sharing. Attach to it,
** find out the base address of heap structures and attach it at the
** correct position.
*/
static int mkattach(struct t_heapmgr **heapp, char keychar)
{
    void *baseaddr, *wanted;
    key_t key = ftok(KEYFILE, keychar);
    int shmid;

    /* Check if the shared memory segment already exists */
    if( (shmid = shmget(key, 0, S_IROTH|S_IWOTH)) ==  -1 )
	return errno;
    if( (baseaddr = shmat(shmid, 0, 0)) == (void *)-1 )
	return errno;
    wanted = ((struct t_heapmgr *)baseaddr)->baseaddr;
    if( wanted != baseaddr ){
	shmdt(baseaddr);
	if( (baseaddr = shmat(shmid, wanted, 0)) == (void *)-1 )
	    return errno;
    }
    *heapp = (struct t_heapmgr *)baseaddr;
    if( fmalock(*heapp) )
	return errno;
    ++(*heapp)->shared;
    fmaunlock(*heapp);
    return 0;
}

int fmainit(size_t len, int flags, char keychar)
{
    struct t_heapmgr heapmgr;
    int rc;

    if( flags & ATTACH )
	return mkattach(&heapmgr_ptr, keychar);

    if( len == 0 || len > MAXMEMORY * PAGESIZE )
	return -2;
    memset(&heapmgr, 0, sizeof heapmgr);
    if( (heapmgr.pagesize = getpagesize()) != PAGESIZE )
	return -1;
    /*
    ** Calculate the page table size. Each entry occupies 4 bytes plus
    ** the heap manager page at the top.
    */
    heapmgr.ptabsz = NUMPAGE(len);
    heapmgr.admin = heapmgr.ptabsz * sizeof(union t_pageentry) + sizeof(heapmgr);
    heapmgr.admin = FITPAGE(heapmgr.admin);
    heapmgr.length = FITPAGE(len);
    len = heapmgr.admin + heapmgr.length;
    if( flags & SYSTEM ){
	if( (heapmgr.baseaddr = malloc(len)) == NULL )
	    return errno;
    }else{
	/* Create a new shared memory segment */
	if( flags & SHARED ){
	    if( rc = mkshared(&heapmgr, len, keychar) )
		return rc;
	}else{
	    if( (heapmgr.shmid = shmget(IPC_PRIVATE, len, IPC_CREAT | IPC_EXCL | S_IRUSR | S_IWUSR)) ==  -1 )
		return errno;
	    if( (heapmgr.baseaddr = shmat(heapmgr.shmid, 0, 0)) == (void *)-1 ){
		shmctl(heapmgr.shmid, IPC_RMID, 0);
		return errno;
	    }
	    heapmgr.shared = 0;
	}
    }
    heapmgr.heapaddr = heapmgr.baseaddr + heapmgr.admin;
    heapmgr.anchor = ENDOFLIST;
    heapmgr.nofree = 0;
    heapmgr.lastused = 0;
    heapmgr.maxused = 0;
    heapmgr.endaddr = (unsigned long)heapmgr.baseaddr + len;
    heapmgr_ptr = heapmgr.baseaddr;
    if( flags & PREFETCH ){
	char *cp = heapmgr.baseaddr;
	for( ; len > 0; len -= PAGESIZE, cp += PAGESIZE )
	    *cp = 0;
    }
    memcpy(heapmgr.baseaddr, &heapmgr, sizeof heapmgr);
    return 0;
}

int fmadown()
{
    key_t shmid = heapmgr_ptr->shmid;
    key_t semid = heapmgr_ptr->semid;
    int shared = heapmgr_ptr->shared;

    printf("usedpages:%d\n", heapmgr_ptr->maxused);
    if( shared <= 1  ){
	semctl(semid, 0, IPC_RMID);
        shmctl(shmid, IPC_RMID, 0);
    }else
	--heapmgr_ptr->shared;
    shmdt(heapmgr_ptr->baseaddr);
    return 0;
}

void *getpage(size_t size)
{
    unsigned long tmp, idx;
    union t_pageentry *pep;
     
    if( heapmgr_ptr->anchor == ENDOFLIST ){
	tmp = (heapmgr_ptr->lastused << PAGESIZE_2) + (unsigned long)heapmgr_ptr->heapaddr;
	if( tmp >= heapmgr_ptr->endaddr || tmp < (unsigned long)heapmgr_ptr->heapaddr )
	    return 0;
	idx = heapmgr_ptr->lastused++;
	if( heapmgr_ptr->maxused < heapmgr_ptr->lastused )
	    heapmgr_ptr->maxused = heapmgr_ptr->lastused;
    }else{
	idx = heapmgr_ptr->anchor;
	heapmgr_ptr->anchor = heapmgr_ptr->ptab[idx].freelist.succ;
	tmp = (unsigned long)heapmgr_ptr->heapaddr + (idx << PAGESIZE_2);
	--heapmgr_ptr->nofree;
    }
    pep = &heapmgr_ptr->ptab[idx];
    pep->pgaddr.slot = size;
    pep->pgaddr.addr = (unsigned long)tmp >> MINSIZE_2;
/*
printf("idx:%d, slot:%d, addr:%x, tmp:%p\n", idx, pep->pgaddr.slot, pep->pgaddr.addr, tmp);
*/
    return (void *)tmp;
}

void *sliceit(struct t_memchunk *here, size_t size, unsigned short cnt)
{
    unsigned long addr = (unsigned long)here;
    struct t_memchunk *here_old;
    do {
	addr += size;
	here->page = (void *)addr;
	here_old = here;
	here = here->page;
    } while( --cnt > 0 );
    here_old->page = NULL;
    return here;
}
