#include	<sys/types.h>
#include	<time.h>
#include	<stdio.h>
#include	<stdlib.h>

#include	"fma.h"

#ifdef SYSTEMHEAP
#define	fmafree free
#define	fmaalloc malloc
#endif

#define ALLOCSIZE 5000
static void *alloc[ALLOCSIZE];
static unsigned long myfreelist[9];
/*                16    32    64    128   256  512  1024 2048 4096 */
float dist[9] = { 0.10, 0.30, 0.30, 0.15, 0.06,0.05,0.02,0.01,0.01 };
unsigned long no = 10, seed = 0, maxmem = 1<<24, maxsize = 4096;

static int mkslot(int size)
{
    int idx;
    if( size <= 16 )
	idx = 0;
    else if( size <= 32 )
	idx = 1;
    else if( size <= 64 )
	idx = 2;
    else if( size <= 128 )
	idx = 3;
    else if( size <= 256 )
	idx = 4;
    else if( size <= 512 )
	idx = 5;
    else if( size <= 1024 )
	idx = 6;
    else if( size <= 2048 )
	idx = 7;
    else if( size <= 4096 )
	idx = 8;
    return idx;
}

int allocate(int callfct)
{
    unsigned long size, maxslot = 0, slot, i, idx;
    long j;
    int drops = 0;

    for( i = 0; i < sizeof myfreelist / sizeof myfreelist[0]; ++i )
	myfreelist[i] = 0;
    for( i = 0; i < ALLOCSIZE; ++i )
	alloc[i] = 0;

    for( i = 0; i < no; ++i){
	do {
	    size = lrand48() % maxsize + 1;
	    slot = mkslot(size);
	}while( (float)myfreelist[slot] / no > dist[slot] );
	myfreelist[slot]++;
	for( j = 0; j < maxslot; ++j )
	    if( alloc[j] == 0 ){
		if( callfct ){
		    if( (alloc[j] = fmaalloc(size)) == NULL ){
			printf("OUT OF MEMORY i:%d, size:%d\n", i, size);
			return;
		    }
		}else
		    alloc[j] = size;
		break;
	    }
	if( j == maxslot ){
	    if( maxslot == ALLOCSIZE ){
		int i1 = 0;
		for(; i1 < ALLOCSIZE; ++i1 ){
		    if( callfct )
			fmafree(alloc[i1]);
		    alloc[i1] = 0;
		}
		maxslot = 0;
		++drops;
	    }
	    if( callfct ){
		if( alloc[maxslot] )
		    printf("USED:alloc[%d]:%x\n", maxslot, alloc[maxslot]);
		if( (alloc[maxslot++] = fmaalloc(size)) == NULL ){
		    printf("OUT OF MEMORY i:%d, size:%d\n", i, size);
		    return;
		}
	    }else
		alloc[maxslot++] = size;
	}
	if( maxslot > 1 && size % 3 == 1 ){
	    j = size % 5;
	    for( ; --j >= 0; ){
		idx = lrand48() % maxslot;
		if( alloc[idx] ){
		    if( callfct )
			fmafree(alloc[idx]);
		    alloc[idx] = 0;
		}
	    }
	}
    }
    for( i = 0 ; i < ALLOCSIZE; ++i ){
	if( alloc[i] ){
	    if( callfct )
		fmafree(alloc[i]);
	}
	alloc[i] = 0;
    }
    return drops;
    for( i = 0 ; i < 8; ++i )
	printf("s(%d) %5d %4.2f\n", 16 << i, myfreelist[i], (float)myfreelist[i]/no);
}

int main(int argc, char **argv)
{
    extern int optind;
    extern char *optarg;
    clock_t start, stop;
    long diff1, diff2;
    unsigned long avg;
    int i, flags = 0;
    
    while( (i = getopt(argc, argv, "s:m:a:f:d:")) != EOF )
	switch( i ){
	case 's':	seed = atol(optarg);
			break;
	case 'f':	flags = atoi(optarg);
			break;
	case 'c':	maxsize = atol(optarg);
			break;
	case 'm':	maxmem = atol(optarg);
			break;
	default:
		fprintf(stderr, "usage: main [-cmaxchunk][-fflags][-sseed][-mmaxmem] [no]\n");
    }
    if( optind < argc )
	no = atoi(argv[optind++]);
    srand48(seed);
    
    start = clock()/1000;
    i = allocate(0);
    stop = clock()/1000;
    diff1 = (stop - start);
    printf("start:%d, stop:%d, diff:%d, drops:%d\n", start, stop, diff1, i);
#if !defined(SYSTEMHEAP)
    if( i = fmainit(maxmem, flags, 'a') ){
	printf("fmainit returned:%d\n", i);
	return 1;
    }
#endif
    sleep(5);
    srand48(seed);
    start = clock()/1000;
    i = allocate(1);
    stop = clock()/1000;
    diff2 = (stop - start);
    printf("start:%d, stop:%d, diff:%d, drops:%d\n", start, stop, diff2, i);
    avg =  diff2 > diff1 ? diff2 - diff1 : 0;
    avg = avg * (1000000 / no);
    printf("allocates: %d difference: %5d ms, average %d ns\n", no, diff2 - diff1, avg);
#if !defined(SYSTEMHEAP)
    fmadown();
#endif
}
